@@ -4,20 +4,19 @@ import getopt
 import json
 import locale
 import os
+import platform
 import re
 import sys
 from urllib import request, parse
-import platform
-import threading
 
 from .version import __version__
-from .util import log, sogou_proxy_server, get_filename, unescape_html
+from .util import log
+from .util.strings import get_filename, unescape_html
 
 dry_run = False
 force = False
 player = None
-sogou_proxy = None
-sogou_env = None
+extractor_proxy = None
 cookies_txt = None
 
 fake_headers = {
@@ -34,10 +33,9 @@ else:
     default_encoding = locale.getpreferredencoding().lower()
 
 def tr(s):
-    try:
-        s.encode(default_encoding)
+    if default_encoding == 'utf-8':
         return s
-    except:
+    else:
         return str(s.encode('utf-8'))[2:-1]
 
 # DEPRECATED in favor of match1()
@@ -112,38 +110,6 @@ def escape_file_path(path):
     path = path.replace('?', '-')
     return path
 
-# DEPRECATED in favor of util.legitimize()
-def filenameable(text):
-    """Converts a string to a legal filename through various OSes.
-    """
-    # All POSIX systems
-    text = text.translate({
-        0: None,
-        ord('/'): '-',
-    })
-    if platform.system() == 'Windows': # For Windows
-        text = text.translate({
-            ord(':'): '-',
-            ord('*'): '-',
-            ord('?'): '-',
-            ord('\\'): '-',
-            ord('\"'): '\'',
-            ord('<'): '-',
-            ord('>'): '-',
-            ord('|'): '-',
-            ord('+'): '-',
-            ord('['): '(',
-            ord(']'): ')',
-        })
-    else:
-        if text.startswith("."):
-            text = text[1:]
-        if platform.system() == 'Darwin': # For Mac OS
-            text = text.translate({
-                ord(':'): '-',
-            })
-    return text
-
 def ungzip(data):
     """Decompresses data for Content-Encoding: gzip.
     """
@@ -236,6 +202,9 @@ def url_size(url, faker = False):
     size = int(response.headers['content-length'])
     return size
 
+# TO BE DEPRECATED
+# urls_size() does not have a faker
+# also it takes too long time
 def urls_size(urls):
     return sum(map(url_size, urls))
 
@@ -257,6 +226,7 @@ def url_info(url, faker = False):
         'video/webm': 'webm',
         'video/x-flv': 'flv',
         'video/x-ms-asf': 'asf',
+        'audio/mp4': 'mp4',
         'audio/mpeg': 'mp3'
     }
     if type in mapping:
@@ -522,13 +492,13 @@ def download_urls(urls, title, ext, total_size, output_dir='.', refer=None, merg
             traceback.print_exc(file = sys.stdout)
             pass
 
-    title = get_filename(title)
+    title = tr(get_filename(title))
 
     filename = '%s.%s' % (title, ext)
     filepath = os.path.join(output_dir, filename)
     if total_size:
         if not force and os.path.exists(filepath) and os.path.getsize(filepath) >= total_size * 0.9:
-            print('Skipping %s: file already exists' % tr(filepath))
+            print('Skipping %s: file already exists' % filepath)
             print()
             return
         bar = SimpleProgressBar(total_size, len(urls))
@@ -555,7 +525,7 @@ def download_urls(urls, title, ext, total_size, output_dir='.', refer=None, merg
         if not merge:
             print()
             return
-        if ext == 'flv':
+        if ext in ['flv', 'f4v']:
             try:
                 from .processor.ffmpeg import has_ffmpeg_installed
                 if has_ffmpeg_installed():
@@ -602,13 +572,13 @@ def download_urls_chunked(urls, title, ext, total_size, output_dir='.', refer=No
 
     assert ext in ('ts')
 
-    title = get_filename(title)
+    title = tr(get_filename(title))
 
     filename = '%s.%s' % (title, 'ts')
     filepath = os.path.join(output_dir, filename)
     if total_size:
         if not force and os.path.exists(filepath[:-3] + '.mkv'):
-            print('Skipping %s: file already exists' % tr(filepath[:-3] + '.mkv'))
+            print('Skipping %s: file already exists' % filepath[:-3] + '.mkv')
             print()
             return
         bar = SimpleProgressBar(total_size, len(urls))
@@ -671,21 +641,22 @@ def download_urls_chunked(urls, title, ext, total_size, output_dir='.', refer=No
 
     print()
 
-def download_rtmp_url(url, playpath, title, ext, total_size=0, output_dir='.', refer=None, merge=True, faker=False):
+def download_rtmp_url(url,title, ext,params={}, total_size=0, output_dir='.', refer=None, merge=True, faker=False):
     assert url
     if dry_run:
         print('Real URL:\n%s\n' % [url])
-        print('Real Playpath:\n%s\n' % [playpath])
+        if params.get("-y",False): #None or unset ->False
+            print('Real Playpath:\n%s\n' % [params.get("-y")])
         return
 
     if player:
         from .processor.rtmpdump import play_rtmpdump_stream
-        play_rtmpdump_stream(player, url, playpath)
+        play_rtmpdump_stream(player, url, params)
         return
 
     from .processor.rtmpdump import has_rtmpdump_installed, download_rtmpdump_stream
     assert has_rtmpdump_installed(), "RTMPDump not installed."
-    download_rtmpdump_stream(url, playpath, title, ext, output_dir)
+    download_rtmpdump_stream(url,  title, ext,params, output_dir)
 
 def playlist_not_supported(name):
     def f(*args, **kwargs):
@@ -736,6 +707,8 @@ def print_info(site_info, title, type, size):
         type_info = "Advanced Systems Format (%s)" % type
     #elif type in ['video/mpeg']:
     #    type_info = "MPEG video (%s)" % type
+    elif type in ['audio/mp4']:
+        type_info = "MPEG-4 audio (%s)" % type
     elif type in ['audio/mpeg']:
         type_info = "MP3 (%s)" % type
     else:
@@ -747,6 +720,18 @@ def print_info(site_info, title, type, size):
     print("Size:      ", round(size / 1048576, 2), "MiB (" + str(size) + " Bytes)")
     print()
 
+def mime_to_container(mime):
+    mapping = {
+        'video/3gpp': '3gp',
+        'video/mp4': 'mp4',
+        'video/webm': 'webm',
+        'video/x-flv': 'flv',
+    }
+    if mime in mapping:
+        return mapping[mime]
+    else:
+        return mime.split('/')[1]
+
 def parse_host(host):
     """Parses host name and port number from a string.
     """
@@ -759,9 +744,6 @@ def parse_host(host):
     port = o.port or 0
     return (hostname, port)
 
-def get_sogou_proxy():
-    return sogou_proxy
-
 def set_proxy(proxy):
     proxy_handler = request.ProxyHandler({
         'http': '%s:%s' % proxy,
@@ -786,7 +768,9 @@ def set_http_proxy(proxy):
     opener = request.build_opener(proxy_support)
     request.install_opener(opener)
 
-def download_main(download, download_playlist, urls, playlist, output_dir, merge, info_only):
+
+
+def download_main(download, download_playlist, urls, playlist, **kwargs):
     for url in urls:
         if url.startswith('https://'):
             url = url[8:]
@@ -794,22 +778,12 @@ def download_main(download, download_playlist, urls, playlist, output_dir, merge
             url = 'http://' + url
 
         if playlist:
-            download_playlist(url, output_dir = output_dir, merge = merge, info_only = info_only)
+            download_playlist(url, **kwargs)
         else:
-            download(url, output_dir = output_dir, merge = merge, info_only = info_only)
-
-def get_version():
-    try:
-        import subprocess
-        real_dir = os.path.dirname(os.path.realpath(__file__))
-        git_hash = subprocess.check_output(['git', 'rev-parse', '--short', 'HEAD'], cwd=real_dir, stderr=subprocess.DEVNULL).decode('utf-8').strip()
-        assert git_hash
-        return '%s-%s' % (__version__, git_hash)
-    except:
-        return __version__
+            download(url, **kwargs)
 
 def script_main(script_name, download, download_playlist = None):
-    version = 'You-Get %s, a video downloader.' % get_version()
+    version = 'You-Get %s, a video downloader.' % __version__
     help = 'Usage: %s [OPTION]... [URL]...\n' % script_name
     help += '''\nStartup options:
     -V | --version                           Display the version and exit.
@@ -821,17 +795,17 @@ def script_main(script_name, download, download_playlist = None):
     -u | --url                               Display the real URLs of videos without downloading.
     -c | --cookies                           Load NetScape's cookies.txt file.
     -n | --no-merge                          Don't merge video parts.
+    -F | --format <STREAM_ID>                Video format code.
     -o | --output-dir <PATH>                 Set the output directory for downloaded videos.
     -p | --player <PLAYER [options]>         Directly play the video with PLAYER like vlc/smplayer.
     -x | --http-proxy <HOST:PORT>            Use specific HTTP proxy for downloading.
+    -y | --extractor-proxy <HOST:PORT>       Use specific HTTP proxy for extracting stream data.
          --no-proxy                          Don't use any proxy. (ignore $http_proxy)
-    -S | --sogou                             Use a Sogou proxy server for downloading.
-         --sogou-proxy <HOST:PORT>           Run a standalone Sogou proxy server.
          --debug                             Show traceback on KeyboardInterrupt.
     '''
 
-    short_opts = 'Vhfiuc:nSo:p:x:'
-    opts = ['version', 'help', 'force', 'info', 'url', 'cookies', 'no-merge', 'no-proxy', 'debug', 'sogou', 'output-dir=', 'player=', 'http-proxy=', 'sogou-proxy=', 'sogou-env=']
+    short_opts = 'Vhfiuc:nF:o:p:x:y:'
+    opts = ['version', 'help', 'force', 'info', 'url', 'cookies', 'no-merge', 'no-proxy', 'debug', 'format=', 'stream=', 'itag=', 'output-dir=', 'player=', 'http-proxy=', 'extractor-proxy=', 'lang=']
     if download_playlist:
         short_opts = 'l' + short_opts
         opts = ['playlist'] + opts
@@ -846,16 +820,18 @@ def script_main(script_name, download, download_playlist = None):
     global force
     global dry_run
     global player
-    global sogou_proxy
-    global sogou_env
+    global extractor_proxy
     global cookies_txt
     cookies_txt = None
 
     info_only = False
     playlist = False
     merge = True
+    stream_id = None
+    lang = None
     output_dir = '.'
     proxy = None
+    extractor_proxy = None
     traceback = False
     for o, a in opts:
         if o in ('-V', '--version'):
@@ -883,44 +859,135 @@ def script_main(script_name, download, download_playlist = None):
             proxy = ''
         elif o in ('--debug',):
             traceback = True
+        elif o in ('-F', '--format', '--stream', '--itag'):
+            stream_id = a
         elif o in ('-o', '--output-dir'):
             output_dir = a
         elif o in ('-p', '--player'):
             player = a
         elif o in ('-x', '--http-proxy'):
             proxy = a
-        elif o in ('-S', '--sogou'):
-            sogou_proxy = ("0.0.0.0", 0)
-        elif o in ('--sogou-proxy',):
-            sogou_proxy = parse_host(a)
-        elif o in ('--sogou-env',):
-            sogou_env = a
+        elif o in ('-y', '--extractor-proxy'):
+            extractor_proxy = a
+        elif o in ('--lang',):
+            lang = a
         else:
             log.e("try 'you-get --help' for more options")
             sys.exit(2)
     if not args:
-        if sogou_proxy is not None:
-            try:
-                if sogou_env is not None:
-                    server = sogou_proxy_server(sogou_proxy, network_env=sogou_env)
-                else:
-                    server = sogou_proxy_server(sogou_proxy)
-                server.serve_forever()
-            except KeyboardInterrupt:
-                if traceback:
-                    raise
-                else:
-                    sys.exit()
-        else:
-            print(help)
-            sys.exit()
+        print(help)
+        sys.exit()
 
     set_http_proxy(proxy)
 
     try:
-        download_main(download, download_playlist, args, playlist, output_dir, merge, info_only)
+        if stream_id:
+            if not extractor_proxy:
+                download_main(download, download_playlist, args, playlist, stream_id=stream_id, output_dir=output_dir, merge=merge, info_only=info_only)
+            else:
+                download_main(download, download_playlist, args, playlist, stream_id=stream_id, extractor_proxy=extractor_proxy, output_dir=output_dir, merge=merge, info_only=info_only)
+        else:
+            if not extractor_proxy:
+                download_main(download, download_playlist, args, playlist, output_dir=output_dir, merge=merge, info_only=info_only)
+            else:
+                download_main(download, download_playlist, args, playlist, extractor_proxy=extractor_proxy, output_dir=output_dir, merge=merge, info_only=info_only)
     except KeyboardInterrupt:
         if traceback:
             raise
         else:
             sys.exit(1)
+
+def url_to_module(url):
+    from .extractors import netease, w56, acfun, baidu, bilibili, blip, catfun, cntv, cbs, coursera, dailymotion, douban, ehow, facebook, freesound, google, sina, ifeng, alive, instagram, iqiyi, joy, jpopsuki, khan, ku6, kugou, kuwo, letv, magisto, miomio, mixcloud, mtv81, nicovideo, pptv, qq, sohu, songtaste, soundcloud, ted, theplatform, tudou, tucao, tumblr, vid48, videobam, vimeo, vine, vk, xiami, yinyuetai, youku, youtube
+
+    video_host = r1(r'https?://([^/]+)/', url)
+    video_url = r1(r'https?://[^/]+(.*)', url)
+    assert video_host and video_url, 'invalid url: ' + url
+
+    if video_host.endswith('.com.cn'):
+        video_host = video_host[:-3]
+    domain = r1(r'(\.[^.]+\.[^.]+)$', video_host) or video_host
+    assert domain, 'unsupported url: ' + url
+
+    k = r1(r'([^.]+)', domain)
+    downloads = {
+        '163': netease,
+        '56': w56,
+        'acfun': acfun,
+        'baidu': baidu,
+        'bilibili': bilibili,
+        'blip': blip,
+        'catfun': catfun,
+        'cntv': cntv,
+        'cbs': cbs,
+        'coursera': coursera,
+        'dailymotion': dailymotion,
+        'douban': douban,
+        'ehow': ehow,
+        'facebook': facebook,
+        'freesound': freesound,
+        'google': google,
+        'iask': sina,
+        'ifeng': ifeng,
+        'in': alive,
+        'instagram': instagram,
+        'iqiyi': iqiyi,
+        'joy': joy,
+        'jpopsuki': jpopsuki,
+        'kankanews': bilibili,
+        'khanacademy': khan,
+        'ku6': ku6,
+        'kugou': kugou,
+        'kuwo': kuwo,
+        'letv': letv,
+        'magisto': magisto,
+        'miomio': miomio,
+        'mixcloud': mixcloud,
+        'mtv81': mtv81,
+        'nicovideo': nicovideo,
+        'pptv': pptv,
+        'qq': qq,
+        'sina': sina,
+        'smgbb': bilibili,
+        'sohu': sohu,
+        'songtaste': songtaste,
+        'soundcloud': soundcloud,
+        'ted': ted,
+        'theplatform': theplatform,
+        "tucao":tucao,
+        'tudou': tudou,
+        'tumblr': tumblr,
+        'vid48': vid48,
+        'videobam': videobam,
+        'vimeo': vimeo,
+        'vine': vine,
+        'vk': vk,
+        'xiami': xiami,
+        'yinyuetai': yinyuetai,
+        'youku': youku,
+        'youtu': youtube,
+        'youtube': youtube,
+    }
+    if k in downloads:
+        return downloads[k], url
+    else:
+        import http.client
+        conn = http.client.HTTPConnection(video_host)
+        conn.request("HEAD", video_url)
+        res = conn.getresponse()
+        location = res.getheader('location')
+        if location is None:
+            raise NotImplementedError(url)
+        else:
+            return url_to_module(location)
+
+def any_download(url, **kwargs):
+    m, url = url_to_module(url)
+    m.download(url, **kwargs)
+
+def any_download_playlist(url, **kwargs):
+    m, url = url_to_module(url)
+    m.download_playlist(url, **kwargs)
+
+def main():
+    script_main('you-get', any_download, any_download_playlist)
